"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const nanoid_1 = __importDefault(require("nanoid"));
const next_config_1 = __importDefault(require("next-server/next-config"));
const constants_1 = require("next-server/constants");
const webpack_config_1 = __importDefault(require("./webpack-config"));
const generate_build_id_1 = require("./generate-build-id");
const write_build_id_1 = require("./write-build-id");
const is_writeable_1 = require("./is-writeable");
const compiler_1 = require("./compiler");
const glob_1 = __importDefault(require("glob"));
const util_1 = require("util");
const querystring_1 = require("querystring");
const glob = util_1.promisify(glob_1.default);
function collectPages(directory, pageExtensions) {
    return glob(`**/*.+(${pageExtensions.join('|')})`, { cwd: directory });
}
async function build(dir, conf = null) {
    if (!await is_writeable_1.isWriteable(dir)) {
        throw new Error('> Build directory is not writeable. https://err.sh/zeit/next.js/build-dir-not-writeable');
    }
    const config = next_config_1.default(constants_1.PHASE_PRODUCTION_BUILD, dir, conf);
    const buildId = await generate_build_id_1.generateBuildId(config.generateBuildId, nanoid_1.default);
    const distDir = path_1.join(dir, config.distDir);
    const pagesDir = path_1.join(dir, 'pages');
    const pagePaths = await collectPages(pagesDir, config.pageExtensions);
    const pages = pagePaths.reduce((result, pagePath) => {
        let page = `/${pagePath.replace(new RegExp(`\\.+(${config.pageExtensions.join('|')})$`), '').replace(/\\/g, '/')}`.replace(/\/index$/, '');
        page = page === '' ? '/' : page;
        result[page] = pagePath;
        return result;
    }, {});
    let entrypoints;
    if (config.target === 'serverless') {
        const serverlessEntrypoints = {};
        // Because on Windows absolute paths in the generated code can break because of numbers, eg 1 in the path,
        // we have to use a private alias
        const pagesDirAlias = 'private-next-pages';
        const dotNextDirAlias = 'private-dot-next';
        const absoluteAppPath = pages['/_app'] ? path_1.join(pagesDirAlias, pages['/_app']).replace(/\\/g, '/') : 'next/dist/pages/_app';
        const absoluteDocumentPath = pages['/_document'] ? path_1.join(pagesDirAlias, pages['/_document']).replace(/\\/g, '/') : 'next/dist/pages/_document';
        const absoluteErrorPath = pages['/_error'] ? path_1.join(pagesDirAlias, pages['/_error']).replace(/\\/g, '/') : 'next/dist/pages/_error';
        const defaultOptions = {
            absoluteAppPath,
            absoluteDocumentPath,
            absoluteErrorPath,
            distDir: dotNextDirAlias,
            buildId,
            assetPrefix: config.assetPrefix,
            generateEtags: config.generateEtags
        };
        Object.keys(pages).forEach(async (page) => {
            if (page === '/_app' || page === '/_document') {
                return;
            }
            const absolutePagePath = path_1.join(pagesDirAlias, pages[page]).replace(/\\/g, '/');
            const bundleFile = page === '/' ? '/index.js' : `${page}.js`;
            const serverlessLoaderOptions = Object.assign({ page, absolutePagePath }, defaultOptions);
            serverlessEntrypoints[path_1.join('pages', bundleFile)] = `next-serverless-loader?${querystring_1.stringify(serverlessLoaderOptions)}!`;
        });
        const errorPage = path_1.join('pages', '/_error.js');
        if (!serverlessEntrypoints[errorPage]) {
            const serverlessLoaderOptions = Object.assign({ page: '/_error', absolutePagePath: 'next/dist/pages/_error' }, defaultOptions);
            serverlessEntrypoints[errorPage] = `next-serverless-loader?${querystring_1.stringify(serverlessLoaderOptions)}!`;
        }
        entrypoints = serverlessEntrypoints;
    }
    const configs = await Promise.all([
        webpack_config_1.default(dir, { buildId, isServer: false, config, target: config.target }),
        webpack_config_1.default(dir, { buildId, isServer: true, config, target: config.target, entrypoints })
    ]);
    let result = { warnings: [], errors: [] };
    if (config.target === 'serverless') {
        const clientResult = await compiler_1.runCompiler([configs[0]]);
        const serverResult = await compiler_1.runCompiler([configs[1]]);
        result = { warnings: [...clientResult.warnings, ...serverResult.warnings], errors: [...clientResult.errors, ...serverResult.errors] };
    }
    else {
        result = await compiler_1.runCompiler(configs);
    }
    if (result.warnings.length > 0) {
        console.warn('> Emitted warnings from webpack');
        result.warnings.forEach((warning) => console.warn(warning));
    }
    if (result.errors.length > 0) {
        result.errors.forEach((error) => console.error(error));
        throw new Error('> Build failed because of webpack errors');
    }
    await write_build_id_1.writeBuildId(distDir, buildId);
}
exports.default = build;
